﻿using System;
using System.Threading.Tasks;
using Microsoft.Bot.Builder.Dialogs;
using Microsoft.Bot.Connector;

namespace smarttalksbot.Dialogs
{
    [Serializable]
    public class NavDialog : IDialog<object>
    {
        public Task StartAsync(IDialogContext context)
        {
            context.Wait(MessageReceivedAsync);

            return Task.CompletedTask;
        }

        private string GetDebugInfo(Activity activity)
        {
            return $"ServiceURL = {activity.ServiceUrl}{Environment.NewLine}{Environment.NewLine}"
                   + $"ChannelId = {activity.ChannelId}{Environment.NewLine}{Environment.NewLine}"
                   + $"ConversationId = {activity.Conversation.Id}{Environment.NewLine}{Environment.NewLine}"
                   + $"RecipientName = {activity.Recipient.Name} {Environment.NewLine}{Environment.NewLine}"
                   + $"RecipientId = {activity.Recipient.Id}{Environment.NewLine}{Environment.NewLine}"
                   + $"SenderId = {activity.From.Id ?? string.Empty}{Environment.NewLine}{Environment.NewLine}"
                   + $"SenderName = {activity.From.Name ?? string.Empty}";
        }

        private string ProcessMessage(Activity activity)
        {
            string responseText = "The bot seems to be upset and doesn't want to answer.";
            try
            {
                string command = activity.Text;

                if (command.ToLower().StartsWith("debug"))
                {
                    responseText = GetDebugInfo(activity);
                }
                else
                {
                    NavTalker navTalker = new NavTalker(activity.From.Id, activity.From.Name);

                    responseText = navTalker.SendMessage2NAV(command);
                }
            }
            catch (Exception ex)
            {
                responseText = $"Exception: {ex.Message}";
                if (ex.InnerException != null)
                {
                    responseText += $"{Environment.NewLine}{Environment.NewLine}Inner Exception: {ex.InnerException.Message}";
                }
            }

            return responseText;
        }

        private async Task MessageReceivedAsync(IDialogContext context, IAwaitable<object> result)
        {
            var activity = await result as Activity;

            // first, send back "typing..."
            Activity typingReply = activity.CreateReply();
            typingReply.Type = ActivityTypes.Typing;
            await context.PostAsync(typingReply);

            // then, start processing command
            string responseText = ProcessMessage(activity);

            // return our reply to the user
            await context.PostAsync(responseText);

            context.Wait(MessageReceivedAsync);
        }
    }
}